﻿using Syncfusion.DocIO.DLS;
using Syncfusion.Windows.Controls.RichTextBoxAdv;
using Syncfusion.Windows.Tools.Controls;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace GettingStarted_RTE_WPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : RibbonWindow
    {
        public MainWindow()
        {
            InitializeComponent();

            //  Loads the file into RichTextBoxAdv
            Assembly execAssm = this.GetType().GetTypeInfo().Assembly;
            Stream stream = execAssm.GetManifestResourceStream("GettingStarted_RTE_WPF.Resources.Getting Started.docx");
            richTextBoxAdv.Load(stream, FormatType.Docx);
        }
    }

    /// <summary>
    /// The list view model.
    /// </summary>
    /// <remarks></remarks>
    public class ListViewModel : INotifyPropertyChanged
    {
        #region Fields
        private string listName;
        private SfRichTextBoxAdv richTextBoxAdv = null;
        Dictionary<string, ListAdv> lists = null;
        private bool isRetrieving = false;
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the ListName.
        /// </summary>
        /// <value></value>
        /// <remarks></remarks>
        public string ListName
        {
            get
            {
                return listName;
            }
            set
            {
                if (value == listName)
                    return;
                listName = value;
                NotifyPropertyChanged("ListName");
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="T:DocumentEditor_WPF.ListViewModel">ListViewModel</see> class. 
        /// </summary>
        /// <param name="richTextBoxAdv"></param>
        /// <remarks></remarks>
        public ListViewModel(SfRichTextBoxAdv richTextBoxAdv)
        {
            if (richTextBoxAdv != null)
            {
                this.richTextBoxAdv = richTextBoxAdv;
                lists = new Dictionary<string, ListAdv>();
                List<string> listNames = new List<string>()
                {
                    "_Bullet_Dot", "_Bullet_Square", "_Bullet_Circle", "_Bullet_Flower", "_Bullet_Tick", "_Bullet_Arrow" ,
                    "_Numbering_LowLetter_Brace","_Numbering_LowLetter_Dot","_Numbering_Number_Brace","_Numbering_Number_Dot","_Numbering_UpLetter","_Numbering_UpRoman","_Numbering_LowRoman",
                };
                foreach (string listName in listNames)
                {
                    lists.Add(listName, CreateList(listName));
                }
                richTextBoxAdv.SelectionChanged += richTextBoxAdv_SelectionChanged;
            }
        }
        #endregion

        #region Events
        /// <summary>
        /// Occurs when Property Changed. 
        /// </summary>
        /// <remarks></remarks>
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// Occurs when the richTextBoxAdv selection changes
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args">An <see cref="T:Syncfusion.Windows.Controls.RichTextBoxAdv.SelectionChangedEventArgs">SelectionChangedEventArgs</see> that contains the event data.</param>
        /// <remarks></remarks>
        void richTextBoxAdv_SelectionChanged(object obj, Syncfusion.Windows.Controls.RichTextBoxAdv.SelectionChangedEventArgs args)
        {
            isRetrieving = true;
            ListAdv list = richTextBoxAdv.Selection.ParagraphFormat.GetList();
            if (list == null)
            {
                if (richTextBoxAdv.Selection.ParagraphFormat.ListLevelNumber == 0)
                    ListName = "NoList";
                else
                    ListName = "Null";
            }
            else
            {
                list = richTextBoxAdv.Selection.Start.Paragraph.ParagraphFormat.ListFormat.List;
                bool hasList = false;
                foreach (string name in lists.Keys)
                {
                    if (lists[name] == list)
                    {
                        ListName = name;
                        hasList = true;
                        break;
                    }
                }
                if (list != null && !hasList)
                {
                    string name = string.Empty;
                    if (list.AbstractList.Levels.Count == 1)
                    {
                        if (richTextBoxAdv.Selection.Start.Paragraph.ParagraphFormat.ListFormat.ListLevel.ListLevelPattern == ListLevelPattern.Bullet)
                        {
                            string bulletType = GetBulletType(richTextBoxAdv.Selection.Start.Paragraph.ParagraphFormat.ListFormat.ListLevel.NumberFormat);
                            name = "_Bullet" + bulletType;
                        }
                        else
                        {
                            string numberFormat = GetNumberingFormat(richTextBoxAdv.Selection.Start.Paragraph.ParagraphFormat.ListFormat.ListLevel);
                            name = "_Numbering" + numberFormat;
                        }
                    }
                    else
                    {
                        if (richTextBoxAdv.Selection.Start.Paragraph.ParagraphFormat.ListFormat.ListLevel.ListLevelPattern == ListLevelPattern.Bullet)
                            name = "_List_Bullet";
                        else
                        {
                            name = "_List";
                            if (richTextBoxAdv.Selection.Start.Paragraph.ParagraphFormat.ListFormat.ListLevel.NumberFormat.Contains("."))
                                name = name + "_Multilevel";
                            else
                                name = name + "_Normal";
                        }
                    }
                    ListName = name;
                }

            }
            isRetrieving = false;
        }
        #endregion

        #region Implementation
        /// <summary>
        /// Notifies when the property changes
        /// </summary>
        /// <param name="propertyName"></param>
        /// <remarks></remarks>
        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            if (!isRetrieving && richTextBoxAdv != null)
            {
                if (listName == "NoList")
                    richTextBoxAdv.Selection.ParagraphFormat.SetList(null);
                if (lists.ContainsKey(ListName))
                    richTextBoxAdv.Selection.ParagraphFormat.SetList(lists[ListName]);
            }
        }
        /// <summary>
        /// Creates the list
        /// </summary>
        /// <param name="listName"></param>
        /// <returns></returns>
        /// <remarks></remarks>
        internal ListAdv CreateList(string listName)
        {
            AbstractListAdv abstractListAdv = new AbstractListAdv(null);
            ListAdv list = new ListAdv(null);
            list.AbstractList = abstractListAdv;
            if (listName.StartsWith("_List"))
            {
                AddListLevels(abstractListAdv, listName);
            }
            else
            {
                ListLevelAdv listLevel = new ListLevelAdv(abstractListAdv);
                abstractListAdv.Levels.Add(listLevel);
                if (listName.StartsWith("_Bullet"))
                {
                    listLevel.ListLevelPattern = ListLevelPattern.Bullet;
                    switch (listName.Substring(8))
                    {
                        case "Dot":
                            listLevel.NumberFormat = "\uf0b7";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Symbol");
                            break;
                        case "Square":
                            listLevel.NumberFormat = "\uf0a7";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                        case "Circle":
                            listLevel.NumberFormat = "\uf06f" + "\u0020";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Symbol");
                            break;
                        case "Flower":
                            listLevel.NumberFormat = "\uf076";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                        case "Arrow":
                            listLevel.NumberFormat = "\uf0d8";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                        case "Tick":
                            listLevel.NumberFormat = "\uf0fc";
                            listLevel.CharacterFormat.FontFamily = new FontFamily("Wingdings");
                            break;
                    }
                }
                else
                {
                    if (listName.Contains("UpRoman"))
                        listLevel.ListLevelPattern = ListLevelPattern.UpRoman;
                    else if (listName.Contains("LowRoman"))
                        listLevel.ListLevelPattern = ListLevelPattern.LowRoman;
                    else if (listName.Contains("UpLetter"))
                        listLevel.ListLevelPattern = ListLevelPattern.UpLetter;
                    else if (listName.Contains("LowLetter"))
                        listLevel.ListLevelPattern = ListLevelPattern.LowLetter;
                    else if (listName.Contains("Number"))
                        listLevel.ListLevelPattern = ListLevelPattern.Number;
                    if (listName.EndsWith("Brace"))
                        listLevel.NumberFormat = "%1)";
                    else
                        listLevel.NumberFormat = "%1.";
                    listLevel.StartAt = 1;
                }
                listLevel.CharacterFormat.FontColor = Colors.Black;
                listLevel.CharacterFormat.FontSize = 14;
                listLevel.ParagraphFormat.LeftIndent = 48;
                listLevel.ParagraphFormat.FirstLineIndent = -24;
            }
            return list;
        }
        /// <summary>
        /// Adds the list levels
        /// </summary>
        /// <param name="abstractListAdv"></param>
        /// <param name="listName"></param>
        /// <remarks></remarks>
        internal void AddListLevels(AbstractListAdv abstractListAdv, string listName)
        {
            List<string> bulletCharacters = new List<string>() { "\uf076", "\uf0d8", "\uf0a7", "\uf0b7", "\uf0a8" };
            for (int i = abstractListAdv.Levels.Count; i < 9; i++)
            {
                ListLevelAdv listLevel = new ListLevelAdv(abstractListAdv);
                if (listName.Contains("Bullet"))
                {
                    listLevel.ListLevelPattern = ListLevelPattern.Bullet;
                    listLevel.NumberFormat = bulletCharacters[i < 5 ? i % 5 : i % 5 + 1];
                    listLevel.CharacterFormat.FontFamily = i < 3 || i == 5 ? new FontFamily("Wingdings") : new FontFamily("Symbol");
                }
                else
                {
                    if (listName.Contains("Multilevel"))
                    {
                        for (int j = 0; j < i + 1; j++)
                            listLevel.NumberFormat += "%" + (j + 1).ToString() + ".";
                        listLevel.ListLevelPattern = ListLevelPattern.Number;
                    }
                    else
                    {
                        listLevel.NumberFormat = "%" + (i + 1).ToString() + ")";
                        listLevel.ListLevelPattern = i % 3 == 0 ? ListLevelPattern.Number
                       : i % 3 == 1 ? ListLevelPattern.LowLetter : ListLevelPattern.LowRoman;
                    }
                    listLevel.StartAt = 1;
                    listLevel.RestartLevel = i;
                }
                listLevel.CharacterFormat.FontColor = Colors.Black;
                listLevel.CharacterFormat.FontSize = 14;
                listLevel.ParagraphFormat.LeftIndent = 48 * i;
                listLevel.ParagraphFormat.FirstLineIndent = -24;
                abstractListAdv.Levels.Add(listLevel);
            }
        }
        /// <summary>
        /// Gets the bullet format.
        /// </summary>
        /// <param name="numberFormat">The number format of bullet pattern type.</param>
        /// <returns>The string representing bullet list format.</returns>
        private string GetBulletType(string numberFormat)
        {
            string bulletType = string.Empty;
            switch (numberFormat)
            {
                case "\uf0b7":
                    bulletType = "_Dot";
                    break;
                case "\uf0a7":
                    bulletType = "_Square";
                    break;
                case "\uf06f" + "\u0020":
                    bulletType = "_Circle";
                    break;
                case "\uf076":
                    bulletType = "_Flower";
                    break;
                case "\uf0d8":
                    bulletType = "_Arrow";
                    break;
                case "\uf0fc":
                    bulletType = "_Tick";
                    break;
            }
            return bulletType;
        }
        /// <summary>
        /// Gets the numbering list format
        /// </summary>
        /// <param name="listLevel">The listlevel.</param>
        /// <returns>The string representing  numbering format.</returns>
        private string GetNumberingFormat(ListLevelAdv listLevel)
        {
            string numberFormat = string.Empty;
            if (listLevel != null)
            {
                if (listLevel.ListLevelPattern == ListLevelPattern.Arabic)
                    numberFormat = "_" + "Number";
                else
                    numberFormat = "_" + listLevel.ListLevelPattern.ToString();

                if (listLevel.NumberFormat == "%1)")
                    numberFormat = numberFormat + "_" + "Brace";
                else
                    numberFormat = numberFormat + "_" + "Dot";
            }
            return numberFormat;
        }

        /// <summary>
        /// Releases all resources used by the.
        /// <see cref="T:DocumentEditor_WPF.ListViewModel">ListViewModel</see>.
        /// </summary>
        /// <remarks></remarks>
        internal void Dispose()
        {
            listName = null;
            if (lists != null)
            {
                lists.Clear();
                lists = null;
            }
            if (richTextBoxAdv != null)
            {
                richTextBoxAdv.SelectionChanged -= richTextBoxAdv_SelectionChanged;
                richTextBoxAdv = null;
            }
        }
        #endregion
    }

    public class SfRichTextBoxAdvExt : SfRichTextBoxAdv
    {

    }

    #region Converters
    /// <summary>
    /// Specifies the List value to Toggle converter.
    /// </summary>
    /// <remarks></remarks>
    internal class ListToggleConverter : IValueConverter
    {
        /// <summary>
        /// Converts the List value to Toggle.
        /// </summary>
        /// <param name="value">The value produced by the binding source.</param>
        /// <param name="targetType">The type of the binding target property.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value != null && parameter != null && value.ToString() == parameter.ToString())
                return true;
            return false;
        }
        /// <summary>
        /// Converts the Toggle value to List.
        /// </summary>
        /// <param name="value">The value that is produced by the binding target.</param>
        /// <param name="targetType">The type to convert to.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (parameter == null)
                return null;
            return parameter.ToString();
        }
    }
    /// <summary>
    /// Specifies the Pixel to Point converter.
    /// </summary>
    public class PixelToPointConverter : IValueConverter
    {

        /// <summary> 
        /// Converts the Pixel value to a string representation of the Point value. 
        /// </summary> 
        /// <param name="value">The value.</param> 
        /// <param name="targetType">The targert type.</param> 
        /// <param name="parameter">The converter parameter</param> 
        /// <param name="language">The language.</param> 
        /// <returns>The converted Point value</returns> 
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            double fontSize = (double)value;
            return ((fontSize * 72) / 96);
        }
        /// <summary> 
        /// Converts string representation of Point to Pixel.  
        /// </summary> 
        /// <param name="value">The value.</param> 
        /// <param name="targetType">The target type.</param> 
        /// <param name="parameter">The converter parameter</param> 
        /// <param name="language">The language.</param> 
        /// <returns>The converted Pixel value.</returns> 
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            double fontSize = 8;
            if (value is string)
            {
                fontSize = double.Parse((string)value);
                return (fontSize * 96) / 72;
            }
            else if (value is double)
            {
                fontSize = (double)value;
                return fontSize;
            }
            return (fontSize * 96) / 72;
        }
    }
    #endregion  
}
